@@ -59,7 +59,7 @@ static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args
 
     PyObject *result = Nuitka_Function_New(
         %(fparse_function_identifier)s,
-        %(mparse_function_identifier)s,
+        %(dparse_function_identifier)s,
         %(function_name_obj)s,
 #if PYTHON_VERSION >= 330
         %(function_qualname_obj)s,
@@ -85,7 +85,7 @@ static PyObject *MAKE_FUNCTION_%(function_identifier)s( %(function_creation_args
 {
     PyObject *result = Nuitka_Function_New(
         %(fparse_function_identifier)s,
-        %(mparse_function_identifier)s,
+        %(dparse_function_identifier)s,
         %(function_name_obj)s,
 #if PYTHON_VERSION >= 330
         %(function_qualname_obj)s,
@@ -216,7 +216,7 @@ catch ( PythonException &_exception )
     frame_guard.getFrame0()->f_locals = %(frame_locals)s;
 #endif
 
-// TODO:
+// Return the error.
     _exception.toPython();
 %(return_code)s
 }"""
